{
 Custom text item for TRVMyTextItemInfo that implements additional properties:
 BorderColor, BorderWidth, AdditionalText.
}

unit CustomTextItem;

interface

uses
  Classes, Graphics, SysUtils,
  RVTypes, RVStyle, RVItem;

type

  TRVMyTextItemInfo = class (TRVTextItemInfo)
  private
    FBorderColor: TColor;
    FAdditionalText: String;
    FBorderWidth: TRVStyleLength;
  public
    constructor Create(ARVData: TPersistent); override;
    procedure Assign(Source: TCustomRVItemInfo); override;
    function AreAllAdditionalPropertiesEqual(Item: TRVTextItemInfo): Boolean; override;
    function GetAdditionalPropertiesAsString: TRVUnicodeString; override;
    function ReadAdditionalPropertiesFromString(const S: TRVUnicodeString): Boolean; override;
    procedure ConvertToDifferentUnits(ARVStyle: TRVStyle;
      OldUnits, NewUnits: TRVStyleUnits; Recursive: Boolean); override;
    property BorderColor: TColor read FBorderColor write FBorderColor;
    property BorderWidth: TRVStyleLength read FBorderWidth write FBorderWidth;
    property AdditionalText: String read FAdditionalText write FAdditionalText;
  end;

implementation

{ TRVMyTextItemInfo }
constructor TRVMyTextItemInfo.Create(ARVData: TPersistent);
begin
  inherited Create(ARVData);
  FBorderColor := clRed;
  FBorderWidth := 0;
  FAdditionalText := '';
end;

procedure TRVMyTextItemInfo.Assign(Source: TCustomRVItemInfo);
begin
  inherited Assign(Source);
  if Source is TRVMyTextItemInfo then
  begin
    BorderColor := TRVMyTextItemInfo(Source).BorderColor;
    BorderWidth := TRVMyTextItemInfo(Source).BorderWidth;
    AdditionalText := TRVMyTextItemInfo(Source).AdditionalText;
  end;
end;


function TRVMyTextItemInfo.AreAllAdditionalPropertiesEqual(
  Item: TRVTextItemInfo): Boolean;
begin
  Result :=
    (BorderColor = TRVMyTextItemInfo(Item).BorderColor) and
    (BorderWidth = TRVMyTextItemInfo(Item).BorderWidth) and
    (AdditionalText = TRVMyTextItemInfo(Item).AdditionalText);
end;


function TRVMyTextItemInfo.GetAdditionalPropertiesAsString: TRVUnicodeString;
var
  SL: TStringList;
begin
  SL := TStringList.Create;
  try
    if BorderColor <> clRed then
      SL.Add('bordercolor=' + IntToStr(BorderColor));
    if BorderWidth <> 0 then
      SL.Add('borderwidth=' + IntToStr(BorderWidth));
    if AdditionalText <> '' then
      SL.Add('additionaltext=' + AdditionalText);
    Result := SL.CommaText;
  finally
    SL.Free;
  end;
end;

function TRVMyTextItemInfo.ReadAdditionalPropertiesFromString(
  const S: TRVUnicodeString): Boolean;
var
  SL: TStringList;
begin
  SL := TStringList.Create;
  try
    SL.CommaText := S;
    BorderColor := StrToIntDef(SL.Values['bordercolor'], clRed);
    BorderWidth := StrToIntDef(SL.Values['borderwidth'], 0);
    AdditionalText := SL.Values['additionaltext'];
  finally
    SL.Free;
  end;
end;
{------------------------------------------------------------------------------}
// This method is needed for items that contains values measured in TRVStyleLength
procedure TRVMyTextItemInfo.ConvertToDifferentUnits(ARVStyle: TRVStyle;
  OldUnits, NewUnits: TRVStyleUnits; Recursive: Boolean);
begin
  inherited ConvertToDifferentUnits(ARVStyle, OldUnits, NewUnits, Recursive);
  BorderWidth := ARVStyle.GetAsDifferentUnitsEx(BorderWidth, OldUnits, NewUnits);
end;
{------------------------------------------------------------------------------}
initialization

{
  After this assignment, TRVMyTextItemInfo becomes the class for text items in
  TRichView
}


  RichViewTextItemClass := TRVMyTextItemInfo;

end.
